/* VIPPR header file
 * 
 * $Id: vippr.h,v 1.4 2001/07/10 04:24:04 fx Exp $
 */
#ifndef _VIPPR_H_
#define _VIPPR_H_

#include <net/ethernet.h>
#include <net/if.h>
#include <netinet/in.h>
#include <signal.h>

#define VIPPR_DEFAULT_CFGFILE		"vippr.conf"
#define STRING_FIX_LENGTH		4096
#define PACKET_MAX			2048

#define ARP_UNKNOWN	0
#define ARP_REQ1	1
#define ARP_REQ2	2
#define ARP_REQ3	3
#define ARP_REQ4	4
#define ARP_REQ5	5
#define ARP_TOREMOVE	6
#define ARP_REVIEW	7
#define TIMER_ARP_REQUESTS	5
#define TIMER_ARP_REVIEW	180

#define VIP_TYPE_STEALTH	0
#define VIP_TYPE_GRE		1
#define VIP_TYPE_8021Q		2
#define VIP_TYPE_PAMN		3

#define VIP_GRE_TUNNELKEY	1

/* Generic structure for packet queues */
typedef struct {
    u_char		*pack;
    unsigned int	plen;
    void		*next;
} queue_t;

/* Real Interfaces 
 */
typedef struct {
    int			sfd;			/* socket on this device */
    char		*name;			/* device name */
    int			idx;			/* device index */
    struct ether_addr	mac;
    struct in_addr	addr;

    queue_t		*outq;

    void		*next;
} interface_t;

/* Virtual ROUTERs
 * 
 * the Router structure is a linked list that holds linked lists. 
 * Every entry is one logical router. This router points to a linked list
 * of route entries. 
 *
 * One routers_t holds many route_t !
 */

typedef struct {
    u_int32_t		dest;
    u_int32_t		mask;
    struct in_addr	gwip;

    void		*next;
} route_t;

typedef struct {
    unsigned int	id;
    route_t		*rtr;

    void		*next;
} routers_t;


/* 
 * VIPs
 */

/* GRE extension */
typedef struct {
    struct in_addr	tunnelsource;
    struct in_addr	tunneldestination;
    unsigned int	flags;
    u_int32_t		tunnelkey;
} ex_gre_t;

/* Virtual IPs
 * 
 * VIP structure is an trival linked list 
 */
typedef struct {
    struct in_addr	addr;
    struct in_addr	mask;
    struct ether_addr	mac;
    unsigned int	router;
    route_t		*rtab;		/* used after initialization */
    interface_t		*intf;		/* points to the interface structure */
    /* extension: type field and speciality union */
    int			viptype;
    union {
	ex_gre_t	gre;
    } ex;

    void		*next;
} vip_t;

/* -- ARP cache */
typedef struct {
    struct in_addr	addr;
    struct ether_addr	mac;
    interface_t		*intf;
    unsigned short	flag;
    unsigned long	tx;

    void		*next;
} arpcache_t;


/* **********************
 * System Section 
 * **********************/

typedef struct {
    int 	verbose;
    int		debug;
    char	*cfgfile;
    int		disable_interface_add;
    int		disable_fastdrop;
} cfg_t;

typedef struct {
    sig_atomic_t	ctrlc;
    u_char		*packet;
    unsigned int	plen;
} sys_t;

/* ******* GLOBAL externs ********** */

extern vip_t		*vip_anchor;
extern routers_t	*routers_anchor;
extern interface_t	*if_anchor;
extern arpcache_t	*arp_anchor;
extern cfg_t		cfg;
extern sys_t		sys;
extern u_char		bcastaddr[];
extern struct in_addr	bcastip;

/* ******* Prototypes ********** */

/* living in vippr_mgmt.c */
int 	vip_add_stealth(char *ipaddr, char *netmask, char *mac, 
		unsigned int router, char *devicename);
int 	vip_add_gre(char *ipaddr, char *netmask, char *mac,
	        unsigned int router, char *devicename,
		char *tunnelsrc, char *tunneldest,
		int usekey, u_int32_t key);
vip_t 	*vip_findbymac(struct ether_addr *mac);
vip_t	*vip_findbymacif(struct ether_addr *mac,interface_t *intf);
vip_t 	*vip_findbyip(struct in_addr *ip);
vip_t	*vip_findbyipif(struct in_addr *ip,interface_t *intf);
void	vip_destroy(void);

/* living in vippr_mgmt.c */
int 	router_add(unsigned int rid);
routers_t *router_findbynum(unsigned int router);
int 	router_add_route(unsigned int rid, char *ip, char *mask, char *gw);
int 	router_linktovip(void);
void	router_print_tables(void);
route_t *router_findnexthop(vip_t *vip, struct in_addr *ip);

/* living in vippr_mgmt.c */
interface_t *interface_add(char *name);
interface_t *interface_findbyname(char *name);
void	interface_destroy(void);
void	interface_send(interface_t *intf, u_char *p, unsigned int len);

/* living in vippr_mgmt.c */
void 	check_arp_table(void);
void	print_arp_table(void);
void	arp_request_all(arpcache_t *a);
void	arp_answer_request(interface_t *intf, struct in_addr *sip,
	        struct ether_addr *smac, struct in_addr *dip);
arpcache_t *arp_new(struct in_addr *ip);
void	arp_remove(arpcache_t *b);
arpcache_t *arp_findbyip(struct in_addr *ip);
arpcache_t *arp_set(interface_t *intf, struct in_addr *ip, 
	        struct ether_addr *mac);

/* living in vippr_net.c */
void	makenonblock(int s);
int	init_device(interface_t *intf);
u_char	*next_packet(int sfd, unsigned int *ptype, int *psize);
void	net_ifrecv(interface_t *intf);
void 	net_ifsend(interface_t *intf);
void	handle_incomming_vip(u_char *packet,unsigned int psize,vip_t *vip);
void	net_route_packet(vip_t *v);
void	net_route_gre(vip_t *v);

/* living in vippr_packs.c */
void	pack_arp_request(vip_t *sender,struct in_addr *who);
void	pack_arp_response(vip_t *sender,struct ether_addr *dmac,
	        struct in_addr *dip);
void 	pack_mketh(u_char *p, vip_t *sender, struct ether_addr *dest,
	        unsigned int ptype);
u_int16_t chksum(u_char *data, unsigned long count);

void yyerror(char *s);
void lamont_hdump(unsigned char *bp, unsigned int length);
void *smalloc(size_t s);
#endif _VIPPR_H_
